@@ -11,7 +11,6 @@
 // limitations under the License.
 
 //! SQL Abstract Syntax Tree (AST) types
-
 mod data_type;
 mod ddl;
 mod operator;
@@ -24,15 +23,15 @@ use alloc::{
     string::{String, ToString},
     vec::Vec,
 };
-use core::fmt;
+use core::fmt::{self, Write};
 
 #[cfg(feature = "serde")]
 use serde::{Deserialize, Serialize};
 
 pub use self::data_type::DataType;
 pub use self::ddl::{
-    AlterTableOperation, ColumnDef, ColumnOption, ColumnOptionDef, ReferentialAction,
-    TableConstraint,
+    AlterColumnOperation, AlterTableOperation, ColumnDef, ColumnOption, ColumnOptionDef,
+    ReferentialAction, TableConstraint,
 };
 pub use self::operator::{BinaryOperator, UnaryOperator};
 pub use self::query::{
@@ -128,7 +127,18 @@ impl From<&str> for Ident {
 impl fmt::Display for Ident {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self.quote_style {
-            Some(q) if q == '"' || q == '\'' || q == '`' => write!(f, "{}{}{}", q, self.value, q),
+            Some(q) if q == '"' || q == '\'' || q == '`' => {
+                f.write_char(q)?;
+                let mut first = true;
+                for s in self.value.split_inclusive(q) {
+                    if !first {
+                        f.write_char(q)?;
+                    }
+                    first = false;
+                    f.write_str(s)?;
+                }
+                f.write_char(q)
+            }
             Some(q) if q == '[' => write!(f, "[{}]", self.value),
             None => f.write_str(&self.value),
             _ => panic!("unexpected quote style"),
@@ -157,16 +167,6 @@ impl fmt::Display for ObjectName {
 pub enum Expr {
     /// Identifier e.g. table name or column name
     Identifier(Ident),
-    /// Unqualified wildcard (`*`). SQL allows this in limited contexts, such as:
-    /// - right after `SELECT` (which is represented as a [SelectItem::Wildcard] instead)
-    /// - or as part of an aggregate function, e.g. `COUNT(*)`,
-    ///
-    /// ...but we currently also accept it in contexts where it doesn't make
-    /// sense, such as `* + *`
-    Wildcard,
-    /// Qualified wildcard, e.g. `alias.*` or `schema.table.*`.
-    /// (Same caveats apply to `QualifiedWildcard` as to `Wildcard`.)
-    QualifiedWildcard(Vec<Ident>),
     /// Multi-part identifier, e.g. `table_alias.column` or `schema.table.col`
     CompoundIdentifier(Vec<Ident>),
     /// `IS NULL` operator
@@ -255,7 +255,7 @@ pub enum Expr {
     },
     MapAccess {
         column: Box<Expr>,
-        keys: Vec<Value>,
+        keys: Vec<Expr>,
     },
     /// Scalar function call e.g. `LEFT(foo, 5)`
     Function(Function),
@@ -284,6 +284,8 @@ pub enum Expr {
     Cube(Vec<Vec<Expr>>),
     /// The `ROLLUP` expr.
     Rollup(Vec<Vec<Expr>>),
+    /// ROW / TUPLE a single value, such as `SELECT (1, 2)`
+    Tuple(Vec<Expr>),
 }
 
 impl fmt::Display for Expr {
@@ -294,15 +296,13 @@ impl fmt::Display for Expr {
                 write!(f, "{}", column)?;
                 for k in keys {
                     match k {
-                        k @ Value::Number(_, _) => write!(f, "[{}]", k)?,
-                        Value::SingleQuotedString(s) => write!(f, "[\"{}\"]", s)?,
+                        k @ Expr::Value(Value::Number(_, _)) => write!(f, "[{}]", k)?,
+                        Expr::Value(Value::SingleQuotedString(s)) => write!(f, "[\"{}\"]", s)?,
                         _ => write!(f, "[{}]", k)?,
                     }
                 }
                 Ok(())
             }
-            Expr::Wildcard => f.write_str("*"),
-            Expr::QualifiedWildcard(q) => write!(f, "{}.*", display_separated(q, ".")),
             Expr::CompoundIdentifier(s) => write!(f, "{}", display_separated(s, ".")),
             Expr::IsNull(ast) => write!(f, "{} IS NULL", ast),
             Expr::IsNotNull(ast) => write!(f, "{} IS NOT NULL", ast),
@@ -447,6 +447,9 @@ impl fmt::Display for Expr {
 
                 write!(f, ")")
             }
+            Expr::Tuple(exprs) => {
+                write!(f, "({})", display_comma_separated(exprs))
+            }
         }
     }
 }
@@ -604,6 +607,22 @@ impl fmt::Display for ShowCreateObject {
     }
 }
 
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+pub enum CommentObject {
+    Column,
+    Table,
+}
+
+impl fmt::Display for CommentObject {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        match self {
+            CommentObject::Column => f.write_str("COLUMN"),
+            CommentObject::Table => f.write_str("TABLE"),
+        }
+    }
+}
+
 /// A top-level statement (SELECT, INSERT, CREATE, etc.)
 #[allow(clippy::large_enum_variant)]
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
@@ -667,6 +686,12 @@ pub enum Statement {
         columns: Vec<Ident>,
         /// VALUES a vector of values to be copied
         values: Vec<Option<String>>,
+        /// file name of the data to be copied from
+        filename: Option<Ident>,
+        /// delimiter character
+        delimiter: Option<Ident>,
+        /// CSV HEADER
+        csv_header: bool,
     },
     /// UPDATE
     Update {
@@ -714,6 +739,8 @@ pub enum Statement {
         query: Option<Box<Query>>,
         without_rowid: bool,
         like: Option<ObjectName>,
+        engine: Option<String>,
+        default_charset: Option<String>,
     },
     /// SQLite's `CREATE VIRTUAL TABLE .. USING <module_name> (<module_args>)`
     CreateVirtualTable {
@@ -786,7 +813,19 @@ pub enum Statement {
     /// `{ BEGIN [ TRANSACTION | WORK ] | START TRANSACTION } ...`
     StartTransaction { modes: Vec<TransactionMode> },
     /// `SET TRANSACTION ...`
-    SetTransaction { modes: Vec<TransactionMode> },
+    SetTransaction {
+        modes: Vec<TransactionMode>,
+        snapshot: Option<Value>,
+        session: bool,
+    },
+    /// `COMMENT ON ...`
+    ///
+    /// Note: this is a PostgreSQL-specific statement.
+    Comment {
+        object_type: CommentObject,
+        object_name: ObjectName,
+        comment: Option<String>,
+    },
     /// `COMMIT [ TRANSACTION | WORK ] [ AND [ NO ] CHAIN ]`
     Commit { chain: bool },
     /// `ROLLBACK [ TRANSACTION | WORK ] [ AND [ NO ] CHAIN ]`
@@ -1028,13 +1067,28 @@ impl fmt::Display for Statement {
                 table_name,
                 columns,
                 values,
+                delimiter,
+                filename,
+                csv_header,
             } => {
                 write!(f, "COPY {}", table_name)?;
                 if !columns.is_empty() {
                     write!(f, " ({})", display_comma_separated(columns))?;
                 }
-                write!(f, " FROM stdin; ")?;
+
+                if let Some(name) = filename {
+                    write!(f, " FROM {}", name)?;
+                } else {
+                    write!(f, " FROM stdin ")?;
+                }
+                if let Some(delimiter) = delimiter {
+                    write!(f, " DELIMITER {}", delimiter)?;
+                }
+                if *csv_header {
+                    write!(f, " CSV HEADER")?;
+                }
                 if !values.is_empty() {
+                    write!(f, ";")?;
                     writeln!(f)?;
                     let mut delim = "";
                     for v in values {
@@ -1047,7 +1101,10 @@ impl fmt::Display for Statement {
                         }
                     }
                 }
-                write!(f, "\n\\.")
+                if filename.is_none() {
+                    write!(f, "\n\\.")?;
+                }
+                Ok(())
             }
             Statement::Update {
                 table,
@@ -1132,6 +1189,8 @@ impl fmt::Display for Statement {
                 query,
                 without_rowid,
                 like,
+                default_charset,
+                engine,
             } => {
                 // We want to allow the following options
                 // Empty column list, allowed by PostgreSQL:
@@ -1257,6 +1316,12 @@ impl fmt::Display for Statement {
                 if let Some(query) = query {
                     write!(f, " AS {}", query)?;
                 }
+                if let Some(engine) = engine {
+                    write!(f, " ENGINE={}", engine)?;
+                }
+                if let Some(default_charset) = default_charset {
+                    write!(f, " DEFAULT CHARSET={}", default_charset)?;
+                }
                 Ok(())
             }
             Statement::CreateVirtualTable {
@@ -1369,11 +1434,22 @@ impl fmt::Display for Statement {
                 }
                 Ok(())
             }
-            Statement::SetTransaction { modes } => {
-                write!(f, "SET TRANSACTION")?;
+            Statement::SetTransaction {
+                modes,
+                snapshot,
+                session,
+            } => {
+                if *session {
+                    write!(f, "SET SESSION CHARACTERISTICS AS TRANSACTION")?;
+                } else {
+                    write!(f, "SET TRANSACTION")?;
+                }
                 if !modes.is_empty() {
                     write!(f, " {}", display_comma_separated(modes))?;
                 }
+                if let Some(snapshot_id) = snapshot {
+                    write!(f, " SNAPSHOT {}", snapshot_id)?;
+                }
                 Ok(())
             }
             Statement::Commit { chain } => {
@@ -1456,6 +1532,18 @@ impl fmt::Display for Statement {
                 }
                 write!(f, "AS {}", statement)
             }
+            Statement::Comment {
+                object_type,
+                object_name,
+                comment,
+            } => {
+                write!(f, "COMMENT ON {} {} IS ", object_type, object_name)?;
+                if let Some(c) = comment {
+                    write!(f, "'{}'", c)
+                } else {
+                    write!(f, "NULL")
+                }
+            }
         }
     }
 }
@@ -1510,7 +1598,7 @@ impl fmt::Display for Privileges {
                 )
             }
             Privileges::Actions(actions) => {
-                write!(f, "{}", display_comma_separated(actions).to_string())
+                write!(f, "{}", display_comma_separated(actions))
             }
         }
     }
@@ -1625,11 +1713,31 @@ impl fmt::Display for Assignment {
     }
 }
 
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+pub enum FunctionArgExpr {
+    Expr(Expr),
+    /// Qualified wildcard, e.g. `alias.*` or `schema.table.*`.
+    QualifiedWildcard(ObjectName),
+    /// An unqualified `*`
+    Wildcard,
+}
+
+impl fmt::Display for FunctionArgExpr {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        match self {
+            FunctionArgExpr::Expr(expr) => write!(f, "{}", expr),
+            FunctionArgExpr::QualifiedWildcard(prefix) => write!(f, "{}.*", prefix),
+            FunctionArgExpr::Wildcard => f.write_str("*"),
+        }
+    }
+}
+
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub enum FunctionArg {
-    Named { name: Ident, arg: Expr },
-    Unnamed(Expr),
+    Named { name: Ident, arg: FunctionArgExpr },
+    Unnamed(FunctionArgExpr),
 }
 
 impl fmt::Display for FunctionArg {
@@ -1815,6 +1923,7 @@ pub enum HiveRowFormat {
     DELIMITED,
 }
 
+#[allow(clippy::large_enum_variant)]
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[allow(clippy::large_enum_variant)]
