@@ -118,6 +118,7 @@ define_keywords!(
     CHAR,
     CHARACTER,
     CHARACTER_LENGTH,
+    CHARSET,
     CHAR_LENGTH,
     CHECK,
     CLOB,
@@ -128,6 +129,7 @@ define_keywords!(
     COLLECT,
     COLUMN,
     COLUMNS,
+    COMMENT,
     COMMIT,
     COMMITTED,
     COMPUTE,
@@ -161,6 +163,7 @@ define_keywords!(
     CURRENT_USER,
     CURSOR,
     CYCLE,
+    DATA,
     DATABASE,
     DATE,
     DAY,
@@ -171,6 +174,7 @@ define_keywords!(
     DEFAULT,
     DELETE,
     DELIMITED,
+    DELIMITER,
     DENSE_RANK,
     DEREF,
     DESC,
@@ -191,6 +195,8 @@ define_keywords!(
     END_EXEC = "END-EXEC",
     END_FRAME,
     END_PARTITION,
+    ENGINE,
+    ENUM,
     EQUALS,
     ERROR,
     ESCAPE,
@@ -412,6 +418,7 @@ define_keywords!(
     SHOW,
     SIMILAR,
     SMALLINT,
+    SNAPSHOT,
     SOME,
     SORT,
     SPECIFIC,
@@ -434,6 +441,7 @@ define_keywords!(
     SUBSTRING_REGEX,
     SUCCEEDS,
     SUM,
+    SUPER,
     SYMMETRIC,
     SYNC,
     SYSTEM,
@@ -468,6 +476,7 @@ define_keywords!(
     TRUE,
     TRUNCATE,
     TRY_CAST,
+    TYPE,
     UESCAPE,
     UNBOUNDED,
     UNCOMMITTED,
